package com.vazil.vridge.docs.service;

import com.vazil.vridge.docs.model.Guide;
import com.vazil.vridge.docs.model.GuideContent;
import com.vazil.vridge.docs.model.GuideTest;
import com.vazil.vridge.docs.repository.GuideContentRepository;
import com.vazil.vridge.docs.repository.GuideRepository;
import com.vazil.vridge.docs.repository.GuideTestRepository;
import com.vazil.vridge.docs.utils.TimeManager;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@AllArgsConstructor
@Log4j2
public class GuideTestService {
    GuideRepository guideRepository;
    GuideTestRepository guideTestRepository;
    GuideContentRepository guideContentRepository;

    public List<GuideTest> findGuideIndex() throws Exception{
//        List<GuideTest> guides = new ArrayList<>();
        List<GuideTest> guideList = guideTestRepository.findAllByOrderByOrder();

//        List<GuideTest> sortedGuideList = guideTestRepository.findAllByParentIdOrderByOrderAsc("");
//        for(GuideTest guide : sortedGuideList) {
//            guide.setChildren(getGuideTreeByParentId(guide.getId()));
//            guides.add(guide);
//        }
        return guideList;
    }

    @Transactional
    public GuideTest findGuideById(String guideId) throws Exception{
        GuideTest targetGuide = guideTestRepository.findById(guideId).orElseThrow(NoSuchElementException::new);

        int viewCount = targetGuide.getView() != null ? targetGuide.getView() : 0;
        targetGuide.setView(viewCount + 1);
        guideTestRepository.save(targetGuide);

//        List<Guide> sortedGuideList = new ArrayList<>();
//        for(Guide guide : guideRepository.findAllByParentIdOrderByOrderAsc("")) {
//            sortedGuideList.add(guide);
//            sortedGuideList.addAll(getGuideListByParentId(guide.getId()));
//        }
//
//        for(int i = 0; i < sortedGuideList.size(); i++) {
//            if(sortedGuideList.get(i).getId().equals(guideId)) {
//                if(i > 0) {
//                    targetGuide.setPrevContent(sortedGuideList.get(i-1));
//                }
//                if(i < sortedGuideList.size() - 1) {
//                    targetGuide.setNextContent(sortedGuideList.get(i+1));
//                }
//                break;
//            }
//        }
        return targetGuide;
    }

    public List<Guide> search(String keyword) throws Exception{
        return guideRepository.findAllByKeywordSetContaining(keyword);
    }

    @Transactional
    public GuideTest createGuide(GuideTest guide) throws Exception{
//        guide.setOrder(guide.getOrder() == null ? getLastOrder(guide.getParentId()) : 0);
        guide.setLike(0);
        guide.setView(0);
        guide.setCreateDate(TimeManager.now());
        return guideTestRepository.save(guide);
    }

    @Transactional
    public Guide updateGuide(Guide guide) throws Exception{
        Guide savedGuide = guideRepository.findById(guide.getId()).orElseThrow(NoSuchElementException::new);

        savedGuide.setParentId(guide.getParentId());
        savedGuide.setOrder(guide.getOrder());
        savedGuide.setUpdateDate(TimeManager.now());
        savedGuide.setContentKey(guide.getContentKey());

        return guideRepository.save(savedGuide);
    }

    @Transactional
    public Guide updateGuideKeyword(Guide guide) throws Exception{
        Guide savedGuide = guideRepository.findById(guide.getId()).orElseThrow(NoSuchElementException::new);
        savedGuide.setKeywordSet(guide.getKeywordSet());
        return guideRepository.save(savedGuide);
    }

    @Transactional
    public Guide deleteGuide(String guideId, boolean cascade) throws Exception{
        Guide guide = guideRepository.findById(guideId).orElseThrow(NoSuchElementException::new);
        String targetId = guide.getId();
        String parentId = guide.getParentId();
        List<Guide> children = getGuideListByParentId(targetId);
        if(cascade) {
            for(Guide g : children) {
                guideRepository.delete(g);
                //guideContentRepository.deleteByGuideId(g.getId());
            }
        } else {
            for(Guide g : guideRepository.findAllByParentIdOrderByOrderAsc(targetId)) {
                g.setParentId(parentId);
                g.setOrder(getLastOrder(parentId));
                guideRepository.save(g);
            }
        }
        guideRepository.delete(guide);
        //guideContentRepository.deleteByGuideId(guide.getId());
        return guide;
    }

    ///////////////////////////////////////////////////////
    // Guide Content CRUD

    @Transactional
    public GuideContent findContentById(String guideId) throws Exception{
        GuideContent targetContent = guideContentRepository.findByGuideId(guideId).orElseThrow(NoSuchElementException::new);
        List<Guide> sortedGuideList = new ArrayList<>();
        for(Guide guide : guideRepository.findAllByParentIdOrderByOrderAsc("")) {
            sortedGuideList.add(guide);
            sortedGuideList.addAll(getGuideListByParentId(guide.getId()));
        }
        for(int i = 0; i < sortedGuideList.size(); i++) {
            if(sortedGuideList.get(i).getId().equals(guideId)) {
                if(i > 0) {
                    targetContent.setPrevContent(sortedGuideList.get(i-1));
                }
                if(i < sortedGuideList.size() - 1) {
                    targetContent.setNextContent(sortedGuideList.get(i+1));
                }
            }
        }
        return  targetContent;
    }

    @Transactional
    public GuideContent updateGuideContent(GuideContent guideContent) throws  Exception{
        GuideContent savedGuideContent = guideContentRepository.findById(guideContent.getId()).orElseThrow(NoSuchElementException::new);

        savedGuideContent.setContent(guideContent.getContent());
        savedGuideContent.setUpdateDate(TimeManager.now());

        return guideContentRepository.save(savedGuideContent);
    }


    /////////////////////////////////////////////////
    // Common

    // 트리 구조로 자식 guide를 가져옴
    private List<Guide> getGuideTreeByParentId(String parentId) {
        List<Guide> children = new ArrayList<>();
        for(Guide childGuide : guideRepository.findAllByParentIdOrderByOrderAsc(parentId)) {
            childGuide.setChildren(guideRepository.findAllByParentIdOrderByOrderAsc(childGuide.getId()));
            children.add(childGuide);
        }
        return children;
    }

    //트리 구조의 guide를 1차원 리스트로 가져옴
    private List<Guide> getGuideListByParentId(String parentId) {
        List<Guide> children = new ArrayList<>();
        for(Guide childGuide : guideRepository.findAllByParentIdOrderByOrderAsc(parentId)) {
            children.add(childGuide);
            children.addAll(guideRepository.findAllByParentIdOrderByOrderAsc(childGuide.getId()));
        }
        return children;
    }

    //현재 뎁스의 마지막 순번을 가져옴
    private Integer getLastOrder(String parentId) {
        return guideRepository.countAllByParentId(parentId);
    }


}
